@@ -297,6 +297,11 @@ enum {
     LD_ECHO			       /* local echo */
 };
 
+enum {
+    /* Actions on remote window title query */
+    TITLE_NONE, TITLE_EMPTY, TITLE_REAL
+};
+
 enum {
     /* Protocol back ends. (cfg.protocol) */
     PROT_RAW, PROT_TELNET, PROT_RLOGIN, PROT_SSH,
@@ -435,8 +440,6 @@ struct config_tag {
     char proxy_telnet_command[512];
     /* SSH options */
     char remote_cmd[512];
-    char remote_cmd2[512];	       /* fallback if the first fails
-					* (used internally for scp) */
     char *remote_cmd_ptr;	       /* might point to a larger command
 				        * but never for loading/saving */
     char *remote_cmd_ptr2;	       /* might point to a larger command
@@ -457,7 +460,7 @@ struct config_tag {
     int try_tis_auth;
     int try_ki_auth;
     int ssh_subsys;		       /* run a subsystem rather than a command */
-    int ssh_subsys2;		       /* fallback to go with remote_cmd2 */
+    int ssh_subsys2;		       /* fallback to go with remote_cmd_ptr2 */
     int ssh_no_shell;		       /* avoid running a shell */
     char ssh_nc_host[512];	       /* host to connect to in `nc' mode */
     int ssh_nc_port;		       /* port to connect to in `nc' mode */
@@ -488,7 +491,7 @@ struct config_tag {
     int no_remote_wintitle;	       /* disable remote retitling */
     int no_dbackspace;		       /* disable destructive backspace */
     int no_remote_charset;	       /* disable remote charset config */
-    int no_remote_qtitle;	       /* disable remote win title query */
+    int remote_qtitle_action;	       /* remote win title query action */
     int app_cursor;
     int app_keypad;
     int nethack_keypad;
@@ -1089,6 +1092,7 @@ typedef struct bidi_char {
 } bidi_char;
 int do_bidi(bidi_char *line, int count);
 int do_shape(bidi_char *line, bidi_char *to, int count);
+int is_rtl(int c);
 
 /*
  * X11 auth mechanisms we know about.
