@@ -294,8 +294,7 @@ void console_provide_logctx(void *logctx)
 
 void logevent(void *frontend, const char *string)
 {
-    if (console_logctx)
-	log_eventlog(console_logctx, string);
+    log_eventlog(console_logctx, string);
 }
 
 static void console_data_untrusted(HANDLE hout, const char *data, int len)
@@ -315,7 +314,7 @@ int console_get_userpass_input(prompts_t *p, unsigned char *in, int inlen)
      */
     {
 	int i;
-	for (i = 0; i < p->n_prompts; i++)
+	for (i = 0; i < (int)p->n_prompts; i++)
 	    memset(p->prompts[i]->result, 0, p->prompts[i]->result_len);
     }
 
